% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3_read_omics.R
\name{extract_rectangle}
\alias{extract_rectangle}
\alias{extract_rectangle.character}
\alias{extract_rectangle.data.table}
\alias{extract_rectangle.matrix}
\title{Extract rectangle from omics file, data.table, or matrix}
\usage{
extract_rectangle(x, ...)

\method{extract_rectangle}{character}(
  x,
  rows = seq_len(nrows(x, sheet = sheet)),
  cols = seq_len(ncols(x, sheet = sheet)),
  verbose = FALSE,
  transpose = FALSE,
  drop = FALSE,
  sheet = 1,
  ...
)

\method{extract_rectangle}{data.table}(
  x,
  rows = seq_len(nrow(x)),
  cols = seq_len(ncol(x)),
  transpose = FALSE,
  drop = FALSE,
  ...
)

\method{extract_rectangle}{matrix}(
  x,
  rows = seq_len(nrow(x)),
  cols = seq_len(ncol(x)),
  transpose = FALSE,
  drop = FALSE,
  ...
)
}
\arguments{
\item{x}{omics datafile or datatable}

\item{...}{allow for S3 method dispatch}

\item{rows}{numeric vector}

\item{cols}{numeric vector}

\item{verbose}{logical}

\item{transpose}{logical}

\item{drop}{logical}

\item{sheet}{numeric or string}
}
\value{
matrix
}
\description{
Extract rectangle from omics file, data.table, or matrix
}
\examples{
# FROM FILE: extract_rectangle.character
#=======================================
# exprs
   require(magrittr)
   x <- download_data('halama18.metabolon.xlsx')
   extract_rectangle(x, rows = 11:401, cols = 15:86, sheet = 2) \%>\%
   extract(1:3, 1:3)

# fids
   extract_rectangle(x, rows = 11:401, cols = 5, sheet = 2)     \%>\%
   extract(1:3, )

# sids
   extract_rectangle(x, rows = 2, cols = 15:86, sheet = 2)      \%>\%
   extract(,1:3)

# fdata
   extract_rectangle(x, rows = 10:401, cols = 1:14,  sheet = 2) \%>\%
   extract(1:3, 1:3)

# sdata
   extract_rectangle(x, rows = 1:10,   cols = 14:86, sheet = 2,
   transpose = TRUE) \%>\% extract(1:3, 1:3)

# FROM MATRIX: extract_rectangle.matrix
#======================================
# exprs
   x <-download_data('halama18.metabolon.xlsx') \%>\%
       extract_rectangle(sheet = 2)
   extract_rectangle(x, rows = 11:401, cols = 15:86, sheet = 2) \%>\%
   extract(1:3, 1:3)

# fids
   extract_rectangle(x, rows = 11:401, cols = 5,     sheet = 2) \%>\%
   extract(1:3, )

# sids
   extract_rectangle(x, rows = 2,      cols = 15:86, sheet = 2) \%>\%
   extract(,1:3)

# fdata
   extract_rectangle(x, rows = 10:401, cols = 1:14,  sheet = 2) \%>\%
   extract(1:3, 1:3)

# sdata
   extract_rectangle(x, rows = 1:10,   cols = 14:86, sheet = 2,
   transpose = TRUE) \%>\% extract(1:3, 1:3)
}
