% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_fit_limma.R, R/8_fit_lmx.R,
%   R/8_fit_wilcoxon.R
\name{fit_limma}
\alias{fit_limma}
\alias{fit_lm}
\alias{fit_lme}
\alias{fit_lmer}
\alias{fit_wilcoxon}
\title{Fit model and test for differential expression}
\usage{
fit_limma(
  object,
  subgroupvar = if ("subgroup" \%in\% svars(object)) "subgroup" else NULL,
  formula = default_formula(object, subgroupvar, "limma"),
  contrastdefs = contrast_coefs(object, formula),
  block = NULL,
  weightvar = if ("weights" \%in\% assayNames(object)) "weights" else NULL,
  verbose = TRUE,
  plot = FALSE
)

fit_lm(
  object,
  subgroupvar = if ("subgroup" \%in\% svars(object)) "subgroup" else NULL,
  formula = default_formula(object, subgroupvar, fit = "lm"),
  block = NULL,
  weightvar = if ("weights" \%in\% assayNames(object)) "weights" else NULL,
  contrastdefs = NULL,
  verbose = TRUE,
  plot = FALSE
)

fit_lme(
  object,
  subgroupvar = if ("subgroup" \%in\% svars(object)) "subgroup" else NULL,
  formula = default_formula(object, subgroupvar, fit = "lme"),
  block = NULL,
  weightvar = if ("weights" \%in\% assayNames(object)) "weights" else NULL,
  contrastdefs = NULL,
  verbose = TRUE,
  plot = FALSE
)

fit_lmer(
  object,
  subgroupvar = if ("subgroup" \%in\% svars(object)) "subgroup" else NULL,
  formula = default_formula(object, subgroupvar, fit = "lmer"),
  block = NULL,
  weightvar = if ("weights" \%in\% assayNames(object)) "weights" else NULL,
  contrastdefs = NULL,
  verbose = TRUE,
  plot = FALSE
)

fit_wilcoxon(
  object,
  subgroupvar = if ("subgroup" \%in\% svars(object)) "subgroup" else NULL,
  formula = default_formula(object, subgroupvar, fit = "wilcoxon"),
  contrastdefs = contrast_coefs(object, formula = formula),
  block = NULL,
  weightvar = NULL,
  verbose = TRUE,
  plot = FALSE
)
}
\arguments{
\item{object}{SummarizedExperiment}

\item{subgroupvar}{subgroup variable}

\item{formula}{modeling formula}

\item{contrastdefs}{contrastdef vector / matrix / list
\itemize{
\item{c("t1-t0", "t2-t1", "t3-t2")}
\item{matrix(c("WT.t1-WT.t0", "WT.t2-WT.t1", "WT.t3-WT.t2"), \cr
     c("KD.t1-KD.t0", "KD.t2-KD.t1", "KD.t3-KD.t2"), nrow=2, byrow=TRUE)}
\item{list(matrix(c("WT.t1-WT.t0", "WT.t2-WT.t1", "WT.t3-WT.t2"), \cr
     c("KD.t1-KD.t0", "KD.t2-KD.t1", "KD.t3-KD.t2"), nrow=2, byrow=TRUE), \cr
     matrix(c("KD.t0-WT.t0", "KD.t1-WT.t1", "KD.t2-WT.t2", "KD.t3-WT.t3"),\cr
     nrow=1, byrow=TRUE))}}}

\item{block}{block svar (or NULL)}

\item{weightvar}{NULL or name of weight matrix in assays(object)}

\item{verbose}{whether to msg}

\item{plot}{whether to plot}
}
\value{
Updated SummarizedExperiment
}
\description{
Fit model and test for differential expression
}
\examples{
require(magrittr)
file <- download_data('atkin18.somascan.adat')
object <- read_somascan(file, plot=FALSE)
object \%<>\% fit_limma(subgroupvar = 'SampleGroup')
object \%<>\% fit_lm(   subgroupvar = 'SampleGroup')
plot_venn(is_sig(object, contrast='t3-t2'))

S4Vectors::metadata(object)$limma <- S4Vectors::metadata(object)$lm <- NULL
object \%<>\% fit_limma(   subgroupvar = 'SampleGroup', block = 'Subject_ID')
object \%<>\% fit_wilcoxon(subgroupvar = 'SampleGroup', block = 'Subject_ID')
# object \%<>\% fit_lme(   subgroupvar = 'SampleGroup', block = 'Subject_ID')
# object \%<>\% fit_lmer(  subgroupvar = 'SampleGroup', block = 'Subject_ID')
plot_venn(is_sig(object, contrast='t3-t2'))
}
