% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_plot.R
\name{plot_boxplots}
\alias{plot_boxplots}
\alias{plot_sample_boxplots}
\alias{plot_feature_boxplots}
\alias{plot_subgroup_boxplots}
\title{Plot sample/feature boxplots}
\usage{
plot_boxplots(
  object,
  x,
  fill,
  color = NULL,
  facet = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_sample_boxplots(
  object,
  x = sample_id,
  fill = sample_id,
  color = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_feature_boxplots(
  object,
  x = feature_id,
  fill = feature_id,
  color = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_subgroup_boxplots(
  object,
  subgroup,
  x = !!enquo(subgroup),
  fill = !!enquo(subgroup),
  color = NULL,
  highlight = NULL,
  facet = feature_id,
  fixed = list(na.rm = TRUE)
)
}
\arguments{
\item{object}{SummarizedExperiment}

\item{x}{svar mapped to x}

\item{fill}{svar mapped to fill}

\item{color}{svar mapped to color}

\item{facet}{svar mapped to facet}

\item{highlight}{fvar expressing which feature should be highlighted}

\item{fixed}{fixed aesthetics}

\item{subgroup}{subgroup svar symbol}
}
\value{
ggplot object
}
\description{
Plot sample/feature boxplots
}
\examples{
# data
    require(magrittr)
    file <- download_data('halama18.metabolon.xlsx')
    object <- read_metabolon(file, plot = FALSE)
    object \%<>\% extract(, order(.$Group))
    fdata(object) \%<>\% cbind(
                        control=.$feature_name \%in\% c('biotin','phosphate'))
# plot
    plot_boxplots(object[1:9,], x = feature_id, fill = feature_id)
    plot_boxplots(object[,1:9], x = sample_id,  fill = sample_id )
    plot_feature_boxplots(object[1:9, ])
    plot_sample_boxplots(object[, 1:12])
    plot_sample_boxplots(object[, 1:12], highlight = control)
    plot_subgroup_boxplots(object[1:2, ], subgroup = Group)
}
\seealso{
\code{\link{plot_sample_densities}},
         \code{\link{plot_sample_violins}}
}
