% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_plot.R
\name{plot_densities}
\alias{plot_densities}
\alias{plot_sample_densities}
\alias{plot_feature_densities}
\title{Plot sample/feature densities}
\usage{
plot_densities(
  object,
  group,
  fill,
  color = NULL,
  fixed = list(alpha = 0.5, na.rm = TRUE)
)

plot_sample_densities(
  object,
  fill = sample_id,
  color = NULL,
  group = sample_id,
  fixed = list(alpha = 0.5, na.rm = TRUE),
  subsetter = if (ncol(object) < 100) {     seq_len(ncol(object)) } else {    
    sample(ncol(object), 9) }
)

plot_feature_densities(
  object,
  fill = feature_id,
  color = NULL,
  group = feature_id,
  fixed = list(alpha = 0.5, na.rm = TRUE),
  subsetter = if (nrow(object) < 100) {     seq_len(nrow(object)) } else {    
    sample(nrow(object), 9) }
)
}
\arguments{
\item{object}{SummarizedExperiment}

\item{group}{svar mapped to group}

\item{fill}{svar mapped to fill}

\item{color}{svar mapped to color}

\item{fixed}{fixed aesthetics}

\item{subsetter}{subsetter for showing a subset of samples/features}
}
\value{
ggplot object
}
\description{
Plot sample/feature densities
}
\examples{
# Read data
    require(magrittr)
    file <- download_data('atkin18.metabolon.xlsx')
    object <- read_metabolon(file, plot = FALSE)
    object \%<>\% extract(, order(.$Group))
# Plot distributions
    plot_sample_densities(object, fill = Group)
    plot_feature_densities(object)
}
\seealso{
\code{\link{plot_sample_violins}},
         \code{\link{plot_sample_boxplots}}
}
