% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7_impute.R
\name{plot_detects}
\alias{plot_detects}
\alias{plot_detections}
\alias{plot_quantifications}
\alias{plot_summarized_detections}
\title{Plot detections}
\usage{
plot_detects(...)

plot_detections(object, subgroup = subgroup, fill = !!enquo(subgroup))

plot_quantifications(...)

plot_summarized_detections(
  object,
  subgroup = subgroup,
  fill = !!enquo(subgroup),
  na_imputes = TRUE
)
}
\arguments{
\item{...}{for backward compatibilty}

\item{object}{SummarizedExperiment}

\item{subgroup}{subgroup var (sym)}

\item{fill}{fill var (sym)}

\item{na_imputes}{whether to NA imputes prior to plottin (TRUE/FALSE)g}
}
\value{
ggplot object
}
\description{
Plot detections
}
\details{
\code{plot_detections} plots feature x sample detections. It shows per
feature/sample nondetects (white), imputes (light colored), and detects
(full color).

\code{plot_summarized_detections} gives an summarized view, plotting
featuretype x subgroup detections. It visualizes the subgroup-wise nondetect
structure often seen in mass spectrometry proteomics data (across e.g.
different cell types)
}
\examples{
require(magrittr)
file <- download_data('fukuda20.proteingroups.txt')
object <- read_proteingroups(file, impute=FALSE, plot = FALSE)
plot_summarized_detections(object)
plot_detections(object)
plot_detections(impute_systematic_nondetects(object, plot=FALSE))

file <- download_data('halama18.metabolon.xlsx')
object <- read_metabolon(file, impute = FALSE, plot = FALSE)
plot_summarized_detections(object, Group)
plot_detections(object, Group)
}
