% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_plot.R
\name{plot_violins}
\alias{plot_violins}
\alias{plot_sample_violins}
\alias{plot_feature_violins}
\alias{plot_subgroup_violins}
\title{Plot sample/feature violins}
\usage{
plot_violins(
  object,
  x,
  fill,
  color = NULL,
  group = NULL,
  facet = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_sample_violins(
  object,
  x = sample_id,
  fill = sample_id,
  color = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_feature_violins(
  object,
  x = feature_id,
  fill = feature_name,
  color = NULL,
  highlight = NULL,
  fixed = list(na.rm = TRUE)
)

plot_subgroup_violins(
  object,
  subgroup,
  x = !!enquo(subgroup),
  fill = !!enquo(subgroup),
  color = NULL,
  highlight = NULL,
  facet = feature_id,
  fixed = list(na.rm = TRUE)
)
}
\arguments{
\item{object}{SummarizedExperiment}

\item{x}{svar mapped to x}

\item{fill}{svar mapped to fill}

\item{color}{svar mapped to color}

\item{group}{svar mapped to group}

\item{facet}{svar mapped to facets}

\item{highlight}{fvar expressing which feature should be highlighted}

\item{fixed}{fixed aesthetics}

\item{subgroup}{subgroup svar}
}
\value{
ggplot object
}
\description{
Plot sample/feature violins
}
\examples{
# data
    require(magrittr)
    file <- download_data('halama18.metabolon.xlsx')
    object <- read_metabolon(file, plot = FALSE)
    object \%<>\% extract(, order(.$Group))
    control_features <- c('biotin','phosphate')
    fdata(object) \%<>\% cbind(control=.$feature_name \%in\% control_features)
# plot
    plot_violins(object[1:12, ], x=feature_id, fill=feature_id)
    plot_feature_violins(object[1:12, ])
    plot_sample_violins(object[, 1:12],  highlight = control)
    plot_subgroup_violins(object[1:4, ], subgroup = Group)
}
\seealso{
\code{\link{plot_sample_densities}},
         \code{\link{plot_sample_boxplots}}
}
