% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3_read_proteingroups.R
\name{.read_maxquant}
\alias{.read_maxquant}
\alias{read_proteingroups}
\alias{read_phosphosites}
\title{Read/Analyze proteingroups/phosphosites}
\usage{
.read_maxquant(
  file,
  quantity = guess_maxquant_quantity(file),
  sfile = NULL,
  sfileby = NULL,
  subgroupvar = "subgroup",
  select_subgroups = NULL,
  invert_subgroups = character(0),
  pepcountpattern = MAXQUANT_PATTERNS_PEPCOUNTS[1],
  verbose = TRUE
)

read_proteingroups(
  file,
  quantity = guess_maxquant_quantity(file),
  sfile = NULL,
  sfileby = NULL,
  select_subgroups = NULL,
  contaminants = FALSE,
  reverse = FALSE,
  fastafile = NULL,
  invert_subgroups = character(0),
  impute = stri_detect_regex(quantity, "[Ii]ntensity"),
  pepcountpattern = MAXQUANT_PATTERNS_PEPCOUNTS[1],
  subgroupvar = NULL,
  formula = NULL,
  block = NULL,
  contrastdefs = NULL,
  pca = FALSE,
  fit = NULL,
  verbose = TRUE,
  plot = TRUE
)

read_phosphosites(
  file,
  proteinfile = paste0(dirname(file), "/proteinGroups.txt"),
  quantity = guess_maxquant_quantity(file),
  sfile = NULL,
  sfileby = NULL,
  select_subgroups = NULL,
  contaminants = FALSE,
  reverse = FALSE,
  min_localization_prob = 0.75,
  fastafile = NULL,
  invert_subgroups = character(0),
  pca = FALSE,
  fit = NULL,
  subgroupvar = NULL,
  formula = NULL,
  block = NULL,
  contrastdefs = NULL,
  verbose = TRUE,
  plot = TRUE
)
}
\arguments{
\item{file}{proteingroups/phosphosites file}

\item{quantity}{string: "Ratio normalized",
"Ratio",
"LFQ intensity",
"Reporter intensity corrected",
"Reporter intensity",
"Intensity labeled",
"Intensity"}

\item{sfile}{sample file}

\item{sfileby}{sample file mergeby column}

\item{subgroupvar}{subgroup svar}

\item{select_subgroups}{subgroups to be selected (character vector)}

\item{invert_subgroups}{subgroups to be inverted (character vector)}

\item{pepcountpattern}{value in MAXQUANT_PATTERNS_PEPCOUNTS}

\item{verbose}{whether to message}

\item{contaminants}{whether to return contaminants}

\item{reverse}{whether to return reverse peptides}

\item{fastafile}{NULL or fastafile (to deconvolute proteingroups)}

\item{impute}{whether to impute consistent nondetects}

\item{formula}{desgnmat formula}

\item{block}{block svar}

\item{contrastdefs}{contrastdef vector/matrix/list}

\item{pca}{whether to pca}

\item{fit}{fit model: NULL, 'limma', 'lm', 'lme', 'lmer','wilcoxon'}

\item{plot}{whether to plot}

\item{proteinfile}{proteingroups file}

\item{min_localization_prob}{min site localization probability (number)}
}
\value{
SummarizedExperiment
}
\description{
Read/Analyze proteingroups/phosphosites
}
\examples{
file <- download_data('fukuda20.proteingroups.txt')
object <- read_proteingroups(file, pca=TRUE, fit='limma')
}
