% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_sumexp_to_dt.R
\name{sumexp_to_wide_dt}
\alias{sumexp_to_wide_dt}
\alias{sumexp_to_long_dt}
\alias{sumexp_to_subrep_dt}
\title{Convert SummarizedExperiment into data.table}
\usage{
sumexp_to_wide_dt(
  object,
  fid = "feature_id",
  fvars = intersect("feature_name", autonomics::fvars(object)),
  assay = assayNames(object)[1]
)

sumexp_to_long_dt(
  object,
  fid = "feature_id",
  fvars = intersect("feature_name", autonomics::fvars(object)),
  sid = "sample_id",
  svars = intersect("subgroup", autonomics::svars(object)),
  assay = assayNames(object) \%>\% intersect(c(.[1], "is_imputed"))
)

sumexp_to_subrep_dt(object, subgroup = subgroup)
}
\arguments{
\item{object}{sumexp}

\item{fid}{fvar carrying feature id}

\item{fvars}{additional fvars to include in table}

\item{assay}{matrix in assays(object) to be used}

\item{sid}{svar carrying sample id}

\item{svars}{additional svars to include in table}

\item{subgroup}{subgroup (sym)}
}
\value{
data.table
}
\description{
Convert SummarizedExperiment into data.table
}
\details{
\itemize{
   \item \code{sumexp_to_wide_dt}:   feature          x sample
   \item \code{sumexp_to_subrep_dt}: feature.subgroup x replicate
   \item \code{sumexp_to_long_dt}:   feature.sample
}
}
\examples{
# Stem cell comparison
    file <- download_data('billing16.proteingroups.txt')
    invert_subgroups <- c('EM_E', 'BM_E', 'EM_BM')
    object <- read_proteingroups(file, invert_subgroups = invert_subgroups,
                  plot=FALSE)
    sumexp_to_wide_dt(object)
    sumexp_to_long_dt(object)
    sumexp_to_subrep_dt(object)

# Glutaminase
   require(magrittr)
   file <- download_data('atkin18.metabolon.xlsx')
   object <- read_metabolon(file, plot=FALSE)
   sumexp_to_wide_dt(object)
   sumexp_to_long_dt(object)
   sumexp_to_subrep_dt(object, Group)

# Fukuda
   require(magrittr)
   file <- download_data('fukuda20.proteingroups.txt')
   object <- read_proteingroups(file, impute=FALSE, plot=FALSE)
   sumexp_to_long_dt(object)
   object \%<>\% impute_systematic_nondetects(plot=FALSE)
   sumexp_to_long_dt(object)
}
